import { Test, TestingModule } from '@nestjs/testing';
import { RedisConfigurationRepository } from '../../adapters/secondaries/redis-configuration.repository';
import { GetConfigurationUseCase } from '../../domain/usecases/get-configuration.usecase';
import { GetConfigurationQuery } from '../../queries/get-configuration.query';

const mockRedisConfigurationRepository = {
  get: jest.fn().mockResolvedValue('my-value'),
};

describe('GetConfigurationUseCase', () => {
  let getConfigurationUseCase: GetConfigurationUseCase;

  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      providers: [
        {
          provide: RedisConfigurationRepository,
          useValue: mockRedisConfigurationRepository,
        },

        GetConfigurationUseCase,
      ],
    }).compile();

    getConfigurationUseCase = module.get<GetConfigurationUseCase>(
      GetConfigurationUseCase,
    );
  });

  it('should be defined', () => {
    expect(getConfigurationUseCase).toBeDefined();
  });

  describe('execute', () => {
    it('should get a value for a key', async () => {
      const value: string = await getConfigurationUseCase.execute(
        new GetConfigurationQuery('my-domain', 'my-key'),
      );

      expect(value).toBe('my-value');
    });
  });
});
