import { PrismaRepository } from '../adapters/secondaries/prisma-repository.abstract';
import { ICollection } from '../interfaces/collection.interface';
import { Point } from './point.type';
import { v4 as uuidv4 } from 'uuid';
import { Territory } from 'src/modules/territory/domain/entities/territory';
import { DatabaseException } from '../exceptions/database.exception';

export class TerritoryRepository<T> extends PrismaRepository<T> {
  async findForPoint(point: Point): Promise<ICollection<T>> {
    return await this.findAllByQuery(
      ['uuid', 'name'],
      [
        `ST_Intersects(ST_GeomFromText('POINT(${point.lon} ${point.lat})',4326),shape) = true`,
      ],
    );
  }

  async findForPoints(points: Point[]): Promise<ICollection<T>> {
    const multipoint = points
      .map((point) => '(' + point.lon + ' ' + point.lat + ')')
      .join(', ');
    return await this.findAllByQuery(
      ['uuid', 'name'],
      [
        `ST_Intersects(ST_GeomFromText('MULTIPOINT(${multipoint})',4326),shape) = true`,
      ],
    );
  }

  async createTerritory(territory: Partial<Territory>): Promise<T> {
    try {
      const affectedRowNumber = await this.createWithFields({
        uuid: `'${uuidv4()}'`,
        name: `'${territory.name}'`,
        shape: `ST_GeomFromGeoJSON('{"type":"MultiPolygon","coordinates":${territory.shape}}')`,
      });
      if (affectedRowNumber == 1) {
        return this.findOne({
          name: territory.name,
        });
      }
      throw new DatabaseException();
    } catch (e) {
      throw e;
    }
  }

  async updateTerritory(
    uuid: string,
    territory: Partial<Territory>,
  ): Promise<T> {
    try {
      const fields = {};
      if (territory.name) fields['name'] = `'${territory.name}'`;
      if (territory.shape)
        fields[
          'shape'
        ] = `ST_GeomFromGeoJSON('{"type":"MultiPolygon","coordinates":${territory.shape}}')`;
      const affectedRowNumber = await this.updateWithFields(uuid, fields);
      if (affectedRowNumber == 1) {
        return this.findOne({
          name: territory.name,
        });
      }
      throw new DatabaseException();
    } catch (e) {
      throw e;
    }
  }
}
