import { Injectable } from '@nestjs/common';
import {
  HealthCheckError,
  HealthIndicator,
  HealthIndicatorResult,
} from '@nestjs/terminus';
import { TerritoriesRepository } from '../../../territory/adapters/secondaries/territories.repository';

@Injectable()
export class PrismaHealthIndicatorUseCase extends HealthIndicator {
  constructor(private readonly _repository: TerritoriesRepository) {
    super();
  }

  async isHealthy(key: string): Promise<HealthIndicatorResult> {
    try {
      await this._repository.healthCheck();
      return this.getStatus(key, true);
    } catch (e) {
      throw new HealthCheckError('Prisma', {
        prisma: e.message,
      });
    }
  }
}
