import { Test, TestingModule } from '@nestjs/testing';
import { TerritoriesRepository } from '../../adapters/secondaries/territories.repository';
import { Messager } from '../../adapters/secondaries/messager';
import { DeleteTerritoryCommand } from '../../commands/delete-territory.command';
import { DeleteTerritoryUseCase } from '../../domain/usecases/delete-territory.usecase';

const mockTerritories = [
  {
    uuid: 'bb281075-1b98-4456-89d6-c643d3044a91',
    name: 'Grand Est',
    shape:
      '[[[[838383.2, 6570485.8],[838483.8, 6570697.7],[838688.1, 6571298.4],[838809.5, 6571599.9],[838383.2, 6570485.8]]]]',
  },
  {
    uuid: 'bb281075-1b98-4456-89d6-c643d3044a92',
    name: 'Nouvelle Aquitaine',
    shape:
      '[[[[838383.2, 6570485.8],[838483.8, 6570697.7],[838688.1, 6571298.4],[838809.5, 6571599.9],[838383.2, 6570485.8]]]]',
  },
  {
    uuid: 'bb281075-1b98-4456-89d6-c643d3044a93',
    name: 'Occitanie',
    shape:
      '[[[[838383.2, 6570485.8],[838483.8, 6570697.7],[838688.1, 6571298.4],[838809.5, 6571599.9],[838383.2, 6570485.8]]]]',
  },
];

const mockTerritoriesRepository = {
  delete: jest
    .fn()
    .mockImplementationOnce((uuid: string) => {
      let savedTerritory = {};
      mockTerritories.forEach((territory, index) => {
        if (territory.uuid === uuid) {
          savedTerritory = { ...territory };
          mockTerritories.splice(index, 1);
        }
      });
      return savedTerritory;
    })
    .mockImplementation(() => {
      throw new Error('Error');
    }),
};

const mockMessager = {
  publish: jest.fn().mockImplementation(),
};

describe('DeleteTerritoryUseCase', () => {
  let deleteTerritoryUseCase: DeleteTerritoryUseCase;

  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      providers: [
        {
          provide: TerritoriesRepository,
          useValue: mockTerritoriesRepository,
        },
        DeleteTerritoryUseCase,
        {
          provide: Messager,
          useValue: mockMessager,
        },
      ],
    }).compile();

    deleteTerritoryUseCase = module.get<DeleteTerritoryUseCase>(
      DeleteTerritoryUseCase,
    );
  });

  it('should be defined', () => {
    expect(deleteTerritoryUseCase).toBeDefined();
  });

  describe('execute', () => {
    it('should delete a territory', async () => {
      const savedUuid = mockTerritories[0].uuid;
      const deleteTerritoryCommand = new DeleteTerritoryCommand(savedUuid);
      await deleteTerritoryUseCase.execute(deleteTerritoryCommand);

      const deletedTerritory = mockTerritories.find(
        (territory) => territory.uuid === savedUuid,
      );
      expect(deletedTerritory).toBeUndefined();
    });
    it('should throw an error if territory does not exist', async () => {
      await expect(
        deleteTerritoryUseCase.execute(
          new DeleteTerritoryCommand('wrong uuid'),
        ),
      ).rejects.toBeInstanceOf(Error);
    });
  });
});
