import { Test, TestingModule } from '@nestjs/testing';
import { TerritoriesRepository } from '../../adapters/secondaries/territories.repository';
import { FindAllTerritoriesRequest } from '../../domain/dtos/find-all-territories.request';
import { FindAllTerritoriesUseCase } from '../../domain/usecases/find-all-territories.usecase';
import { FindAllTerritoriesQuery } from '../../queries/find-all-territories.query';

const findAllTerritoriesRequest: FindAllTerritoriesRequest =
  new FindAllTerritoriesRequest();
findAllTerritoriesRequest.page = 1;
findAllTerritoriesRequest.perPage = 10;

const findAllTerritoriesQuery: FindAllTerritoriesQuery =
  new FindAllTerritoriesQuery(findAllTerritoriesRequest);

const mockTerritories = [
  {
    uuid: 'bb281075-1b98-4456-89d6-c643d3044a91',
    name: 'Grand Est',
    shape:
      '[[[[838383.2, 6570485.8],[838483.8, 6570697.7],[838688.1, 6571298.4],[838809.5, 6571599.9],[838383.2, 6570485.8]]]]',
  },
  {
    uuid: 'bb281075-1b98-4456-89d6-c643d3044a92',
    name: 'Nouvelle Aquitaine',
    shape:
      '[[[[838383.2, 6570485.8],[838483.8, 6570697.7],[838688.1, 6571298.4],[838809.5, 6571599.9],[838383.2, 6570485.8]]]]',
  },
  {
    uuid: 'bb281075-1b98-4456-89d6-c643d3044a93',
    name: 'Occitanie',
    shape:
      '[[[[838383.2, 6570485.8],[838483.8, 6570697.7],[838688.1, 6571298.4],[838809.5, 6571599.9],[838383.2, 6570485.8]]]]',
  },
];

const mockTerritoriesRepository = {
  // eslint-disable-next-line @typescript-eslint/no-unused-vars
  findAll: jest.fn().mockImplementation((query?: FindAllTerritoriesQuery) => {
    return Promise.resolve(mockTerritories);
  }),
};

describe('FindAllTerritoriesUseCase', () => {
  let findAllTerritoriesUseCase: FindAllTerritoriesUseCase;

  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      providers: [
        {
          provide: TerritoriesRepository,
          useValue: mockTerritoriesRepository,
        },
        FindAllTerritoriesUseCase,
      ],
    }).compile();

    findAllTerritoriesUseCase = module.get<FindAllTerritoriesUseCase>(
      FindAllTerritoriesUseCase,
    );
  });

  it('should be defined', () => {
    expect(findAllTerritoriesUseCase).toBeDefined();
  });

  describe('execute', () => {
    it('should return an array filled with territories', async () => {
      const territories = await findAllTerritoriesUseCase.execute(
        findAllTerritoriesQuery,
      );

      expect(territories).toBe(mockTerritories);
    });
  });
});
